/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import java.io.*;
import java.security.*;
import java.security.KeyStore.Entry;
import java.security.KeyStore.PasswordProtection;
import java.security.KeyStore.PrivateKeyEntry;
import java.security.KeyStore.TrustedCertificateEntry;
import java.security.cert.*;
import java.security.cert.Certificate;
import java.util.Arrays;
import javax.annotation.*;
import javax.annotation.concurrent.*;

import net.java.truelicense.core.codec.Codec;
import net.java.truelicense.core.io.Source;
import static net.java.truelicense.core.io.Store.BUFSIZE;
import net.java.truelicense.core.util.*;
import net.java.truelicense.obfuscate.*;

/**
 * Signs or verifies {@linkplain Repository repositories} with an entry in a
 * key store.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public final class Notary implements Authentication {

    @Obfuscate
    private static final String DEFAULT_ALGORITHM = "SHA1withDSA";

    @Obfuscate
    static final String NO_STORE_PASSWORD = "noStorePassword";

    @Obfuscate
    static final String NO_PRIVATE_KEY = "noPrivateKey";

    @Obfuscate
    static final String NO_CERTIFICATE = "noCertificate";

    @Obfuscate
    static final String MISSING_PASSWORD = "missingPassword";

    @Obfuscate
    static final String SUPERFLUOUS_PASSWORD = "superfluousPassword";

    @Obfuscate
    static final String NO_SUCH_ENTRY = "noSuchEntry";

    private final KeyStoreParameters ksp;

    public Notary(final KeyStoreParameters ksp) {
        this.ksp = Objects.requireNonNull(ksp);
    }

    @Override public KeyStoreParameters parameters() { return ksp; }

    private Source source() { return parameters().source(); }

    private String storeType() { return parameters().storeType(); }

    private char[] storePassword() { return parameters().storePassword(); }

    private String alias() { return parameters().alias(); }

    private @CheckForNull char[] keyPassword() {
        return parameters().keyPassword();
    }

    private @Nullable InputStream input() throws IOException {
        final Source source = source();
        return null == source ? null : source.input();
    }

    @Override public Artifactory sign(
            final Codec codec,
            final Repository repository,
            final @Nullable Object artifact)
    throws Exception {
        return repository.sign(codec, engine(), privateKey(), artifact);
    }

    @Override public Artifactory verify(
            final Codec codec,
            final Repository repository)
    throws Exception {
        return repository.verify(codec, engine(), publicKey());
    }

    private Signature engine() throws Exception {
        return Signature.getInstance(algorithm());
    }

    private String algorithm() throws Exception {
        final Certificate cert = certificate();
        if (cert instanceof X509Certificate) {
            return ((X509Certificate) cert).getSigAlgName();
        } else {
            return DEFAULT_ALGORITHM;
        }
    }

    private PrivateKey privateKey() throws Exception {
        final Entry entry = entry();
        if (entry instanceof PrivateKeyEntry) {
            return ((PrivateKeyEntry) entry).getPrivateKey();
        } else {
            throw new NotaryException(message(NO_PRIVATE_KEY));
        }
    }

    private PublicKey publicKey() throws Exception {
        return certificate().getPublicKey();
    }

    private Certificate certificate() throws Exception {
        final Entry entry = entry();
        if (entry instanceof PrivateKeyEntry) {
            return ((PrivateKeyEntry) entry).getCertificate();
        } else if (entry instanceof TrustedCertificateEntry) {
            return ((TrustedCertificateEntry) entry).getTrustedCertificate();
        } else {
            throw new NotaryException(message(NO_CERTIFICATE));
        }
    }

    private Entry entry() throws Exception {
        final KeyStore store = keyStore();
        final String alias = alias();
        final char[] pwd = keyPassword();
        if (null != pwd) {
            try {
                if (store.isKeyEntry(alias)) {
                    final PasswordProtection pp = new PasswordProtection(pwd);
                    try {
                        return store.getEntry(alias, pp);
                    } finally {
                        pp.destroy();
                    }
                }
            } finally {
                Arrays.fill(pwd, (char) 0);
            }
            if (store.isCertificateEntry(alias))
                throw new NotaryException(message(SUPERFLUOUS_PASSWORD));
        } else {
            if (store.isCertificateEntry(alias))
                return store.getEntry(alias, null);
            if (store.isKeyEntry(alias))
                throw new NotaryException(message(MISSING_PASSWORD));
        }
        assert !store.containsAlias(alias);
        throw new NotaryException(message(NO_SUCH_ENTRY));
    }

    private Message message(String key) {
        return Messages.message(key, alias());
    }

    /**
     * Returns the loaded key store.
     *
     * @throws GeneralSecurityException if there is any problem accessing the
     *         key store data.
     */
    private KeyStore keyStore() throws Exception {
        final char[] pwd = storePassword();
        if (null == pwd) throw new NotaryException(message(NO_STORE_PASSWORD));
        try {
            final KeyStore store = KeyStore.getInstance(storeType());
            final InputStream in = new BufferedInputStream(input(), BUFSIZE);
            try { store.load(in, pwd); }
            finally { in.close(); }
            return store;
        } finally {
            Arrays.fill(pwd, (char) 0);
        }
    }
}
